@@ -25,6 +25,7 @@
 #' @param i.level.other Level of confidence interval to calculate length, start and percentages.
 #' @param i.method Method to calculate the optimal timing of the epidemic.
 #' @param i.param Parameter to calculate the optimal timing of the epidemic.
+#' @param i.centering Number of weeks to center the moving seasons.
 #' @param i.n.max Number of pre-epidemic values used to calculate the threshold.
 #' @param i.type.boot Type of bootstrap technique.
 #' @param i.iter.boot Number of bootstrap iterations.
@@ -64,12 +65,17 @@
 #' \itemize{
 #' \item{1} {Arithmetic mean and mean confidence interval.}
 #' \item{2} {Geometric mean and mean confidence interval.}
-#' \item{3} {Median and the KC Method to calculate its confidence interval.}
+#' \item{3} {Median and the Nyblom/normal aproximation confidence interval.}
 #' \item{4} {Median and bootstrap confidence interval.}
 #' \item{5} {Arithmetic mean and point confidence interval (standard deviations).}
 #' \item{6} {Geometric mean and point confidence interval (standard deviations).}
 #' }
 #'
+#' Option \code{3} uses the Hettmansperger and Sheather (1986) and Nyblom (1992) method, 
+#' when there is enough sample size. If sample size is small, then the normal aproximation
+#' will be used as described in Conover, 1980, p. 112. Refer to EnvStats package for
+#' more information.
+#'
 #' Option \code{4} uses two more parameters: \code{i.type.boot} indicates which bootstrap
 #' method to use. The values are the same of those of the \code{\link{boot.ci}} function.
 #' Parameter \code{i.iter.boot} indicates the number of bootstrap samples to use. See
@@ -155,6 +161,14 @@
 #' Lozano JE. lozalojo/mem: Second release of the MEM R library. Zenodo [Internet]. [cited 2017 Feb 1];
 #' Available from: \url{https://zenodo.org/record/165983}. DOI:10.5281/zenodo.165983
 #'
+#' Hettmansperger, T. P., and S. J Sheather. 1986. Confidence Intervals Based on Interpolated Order
+#' Statistics. Statistics and Probability Letters 4: 75-79. doi:10.1016/0167-7152(86)90021-0.
+#'
+#' Nyblom, J. 1992. Note on Interpolated Order Statistics. Statistics and Probability Letters 14:
+#' 129-31. doi:10.1016/0167-7152(92)90076-H.
+#'
+#' Conover, W.J. (1980). Practical Nonparametric Statistics. Second Edition. John Wiley and Sons, New York.
+#'
 #' @keywords influenza
 #'
 #' @export
@@ -176,30 +190,31 @@ memmodel<-function(i.data,
                    i.level.other=0.95,
                    i.method=2,
                    i.param=2.8,
+                   i.centering=-1,
                    i.n.max=-1,
                    i.type.boot="norm",
                    i.iter.boot=10000){
-
+  
   if (is.null(dim(i.data))) stop('Incorrect number of dimensions, input must be a data.frame.') else if (!(ncol(i.data)>1)) stop('Incorrect number of dimensions, at least two seasons of data required.')
-
+  
   datos<-i.data[apply(i.data,2,function(x) sum(x,na.rm=T)>0)]
-
+  
   if (NCOL(datos)==0){
-
+    
     memmodel.output <- NULL
-
+    
   }else{
     if (is.matrix(datos)) datos<-as.data.frame(datos)
-
+    
     if (!is.na(i.seasons)) if (i.seasons>0) datos<-datos[(max((dim(datos)[2])-i.seasons+1,1)):(dim(datos)[2])]
-
+    
     datos<-as.data.frame(apply(datos,2,fill.missing))
-
+    
     semanas<-dim(datos)[1]
     anios<-NCOL(datos)
-
+    
     # Calcular el optimo
-
+    
     if (is.na(i.n.max)){
       n.max<-max(1,round(30/anios,0))
     }else{
@@ -212,29 +227,46 @@ memmodel<-function(i.data,
         n.max=i.n.max
       }
     }
-
+    
     optimo<-apply(datos,2,memtiming,i.n.values=n.max,i.method=i.method,i.param=i.param)
-
+    
     datos.duracion.real<-extraer.datos.optimo.map(optimo)
-
+    
     # por defecto la aritmetica en ambos
     ic.duracion<-iconfianza(as.numeric(datos.duracion.real[1,]),i.level.other,i.type.other,T,i.type.boot,i.iter.boot,2)
     ic.duracion<-rbind(ic.duracion,c(floor(ic.duracion[1]),round(ic.duracion[2]),ceiling(ic.duracion[3])))
     ic.porcentaje<-iconfianza(as.numeric(datos.duracion.real[2,]),i.level.other,i.type.other,T,i.type.boot,i.iter.boot,2)
+    duracion.media <- ic.duracion[2,2]
 
-    duracion.media<-ic.duracion[2,2]
-
+    if (is.na(i.centering) | i.centering==-1) duracion.centrado <- duracion.media else duracion.centrado <- i.centering
+    
     ########################################################################################
     # Calcula todos los parametros asociados a la temporada gripal en base a la estimacion #
     # del numero de semanas de duracion total.                                             #
     ########################################################################################
-
+    
     # Datos de la gripe en la temporada REAL y en la temporada MEDIA, esta ultima sirve para unir las temporadas.
-
+    
     semana.inicio<-as.integer(rownames(datos)[1])
-    gripe<-array(dim=c(7,anios,2))
-
-    datos.duracion.media<-extraer.datos.curva.map(optimo,duracion.media)
+    gripe<-array(dim=c(7, anios, 3), dimnames = list("indicators"=c("epidemic start", "epidemic end", "sum of epidemic values", "sum of values", "covered percentage", "sum of pre-epidemic values", "sum of post-epidemic values"), 
+                                                     "seasons"=names(datos), 
+                                                     "type"=c("real epidemic", "mean length epidemic", "centered length epidemic")))
+    datos.duracion.media <- extraer.datos.curva.map(optimo, duracion.media)
+    datos.duracion.centrado <- extraer.datos.curva.map(optimo, duracion.centrado)
+    
+    ## Estimacion del numero de semana en el  comienza la gripe
+    
+    ic.inicio<-iconfianza(as.numeric(datos.duracion.real[4,]),i.level.other,i.type.other,T,i.type.boot,i.iter.boot,2)
+    ic.inicio<-rbind(ic.inicio,c(floor(ic.inicio[1]),round(ic.inicio[2]),ceiling(ic.inicio[3])))
+    ic.inicio<-rbind(ic.inicio,c(semana.absoluta(ic.inicio[2,1],semana.inicio),semana.absoluta(ic.inicio[2,2],semana.inicio),semana.absoluta(ic.inicio[2,3],semana.inicio)))
+    ic.inicio<-ic.inicio[c(2,3,1),]
+    inicio.medio<-ic.inicio[1,2]
+    
+    ic.inicio.centrado<-iconfianza(as.numeric(datos.duracion.centrado[4,]),i.level.other,i.type.other,T,i.type.boot,i.iter.boot,2)
+    ic.inicio.centrado<-rbind(ic.inicio.centrado,c(floor(ic.inicio.centrado[1]),round(ic.inicio.centrado[2]),ceiling(ic.inicio.centrado[3])))
+    ic.inicio.centrado<-rbind(ic.inicio.centrado,c(semana.absoluta(ic.inicio.centrado[2,1],semana.inicio),semana.absoluta(ic.inicio.centrado[2,2],semana.inicio),semana.absoluta(ic.inicio.centrado[2,3],semana.inicio)))
+    ic.inicio.centrado<-ic.inicio.centrado[c(2,3,1),]
+    inicio.centrado<-ic.inicio.centrado[1,2]
 
     for (j in 1:anios){
       ## Semana en la q comienza la temporada.real
@@ -251,134 +283,136 @@ memmodel<-function(i.data,
       if (gripe[1,j,1]>1) gripe[6,j,1]<-sum(datos[1:(gripe[1,j,1]-1),j],na.rm=TRUE) else gripe[6,j,1]<-0
       ## Casos Despues de la temporada gripal
       if (gripe[2,j,1]<semanas) gripe[7,j,1]<-sum(datos[(gripe[2,j,1]+1):semanas,j],na.rm=TRUE) else gripe[7,j,1]<-0
-
-      ## Semana en la q comienza la temporada.media
+      
       gripe[1,j,2]<-datos.duracion.media[4,j]
-      ## Semana en la q acaba
       gripe[2,j,2]<-datos.duracion.media[5,j]
-      ## numero de casos en la temporada.media de gripe
       gripe[3,j,2]<-datos.duracion.media[3,j]
-      ## Numero de casos totales de la temporada global
       gripe[4,j,2]<-sum(datos[,j],na.rm=TRUE)
-      ## Porcentaje cubierto de casos de la temporada gripal
       gripe[5,j,2]<-datos.duracion.media[2,j]
-      ## Casos Acumulados justo antes de comenzar la temporada gripal
       if (gripe[1,j,2]>1) gripe[6,j,2]<-sum(datos[1:(gripe[1,j,2]-1),j],na.rm=TRUE) else gripe[6,j,2]<-0
-      ## Casos Despues de la temporada gripal
       if (gripe[2,j,2]<semanas) gripe[7,j,2]<-sum(datos[(gripe[2,j,2]+1):semanas,j],na.rm=TRUE) else gripe[7,j,2]<-0
+      
+      gripe[1,j,3]<-datos.duracion.centrado[4,j]
+      gripe[2,j,3]<-datos.duracion.centrado[5,j]
+      gripe[3,j,3]<-datos.duracion.centrado[3,j]
+      gripe[4,j,3]<-sum(datos[,j],na.rm=TRUE)
+      gripe[5,j,3]<-datos.duracion.centrado[2,j]
+      if (gripe[1,j,3]>1) gripe[6,j,3]<-sum(datos[1:(gripe[1,j,3]-1),j],na.rm=TRUE) else gripe[6,j,3]<-0
+      if (gripe[2,j,3]<semanas) gripe[7,j,3]<-sum(datos[(gripe[2,j,3]+1):semanas,j],na.rm=TRUE) else gripe[7,j,3]<-0
     }
-
+    
     # Indice de estado temporal. Si es un 1 me dice que estamos en epoca pretemporada,
     # si pone un 2 indica que estamos en temporada gripal. Si pone un 3 indica que
     # estamos en posttemporada.
-
-    indices.temporada<-array(dim=c(semanas,anios,2))
-
+    
+    indices.temporada<-array(dim=c(semanas, anios, 3), dimnames = list("weeks"=rownames(datos), 
+                                                                       "seasons"=names(datos), 
+                                                                       "type"=c("real epidemic", "mean length epidemic", "centered length epidemic")))
     for (j in 1:anios){
       indices.temporada[-((datos.duracion.real[4,j]):semanas),j,1]<-1
       indices.temporada[(datos.duracion.real[4,j]):(datos.duracion.real[5,j]),j,1]<-2
       indices.temporada[-(1:(datos.duracion.real[5,j])),j,1]<-3
+      
       indices.temporada[-((datos.duracion.media[4,j]):semanas),j,2]<-1
       indices.temporada[(datos.duracion.media[4,j]):(datos.duracion.media[5,j]),j,2]<-2
       indices.temporada[-(1:(datos.duracion.media[5,j])),j,2]<-3
+      
+      indices.temporada[-((datos.duracion.centrado[4,j]):semanas),j,3]<-1
+      indices.temporada[(datos.duracion.centrado[4,j]):(datos.duracion.centrado[5,j]),j,3]<-2
+      indices.temporada[-(1:(datos.duracion.centrado[5,j])),j,3]<-3
     }
-
-    ## Estimacion del numero de semana en el  comienza la gripe
-
-    ic.inicio<-iconfianza(as.numeric(datos.duracion.real[4,]),i.level.other,i.type.other,T,i.type.boot,i.iter.boot,2)
-    ic.inicio<-rbind(ic.inicio,c(floor(ic.inicio[1]),round(ic.inicio[2]),ceiling(ic.inicio[3])))
-    ic.inicio<-rbind(ic.inicio,c(semana.absoluta(ic.inicio[2,1],semana.inicio),semana.absoluta(ic.inicio[2,2],semana.inicio),semana.absoluta(ic.inicio[2,3],semana.inicio)))
-    ic.inicio<-ic.inicio[c(2,3,1),]
-    inicio.medio<-ic.inicio[1,2]
-
+    
+    
     # Grafico del esquema de las temporadas gripales para su union
-
-    longitud.esquema<-semanas+max.fix.na(gripe[1,,2])-min.fix.na(gripe[1,,2])
-    inicio.epidemia.esquema<-max.fix.na(gripe[1,,2])
-    fin.epidemia.esquema<-max.fix.na(gripe[2,,2])
-
-    esquema.temporadas<-array(dim=c(longitud.esquema,anios+2,3))
-
+    
+    longitud.esquema <- semanas + max.fix.na(datos.duracion.centrado[4,]) - min.fix.na(datos.duracion.centrado[4,])
+    inicio.epidemia.esquema<-max.fix.na(datos.duracion.centrado[4,])
+    fin.epidemia.esquema<-max.fix.na(datos.duracion.centrado[5,])
+    
+    esquema.temporadas<-array(dim=c(longitud.esquema, anios+2, 3), dimnames = list("dummy weeks"=as.character(1:longitud.esquema), 
+                                                                                   "seasons"=c(names(datos), "mean season week", "mean season index"), 
+                                                                                   "type"=c("absolute week", "relative week", "values")))
+    
     for (j in 1:anios){
+      diferencia.anual <- inicio.epidemia.esquema - datos.duracion.centrado[4,j]
       for (i in 1:semanas){
-        diferencia<-inicio.epidemia.esquema-gripe[1,j,2]
-        esquema.temporadas[i+diferencia,j,1]<-i
-        esquema.temporadas[i+diferencia,j,2]<-semana.absoluta(i,semana.inicio)
-        esquema.temporadas[i+diferencia,j,3]<-datos[i,j]
+        esquema.temporadas[i + diferencia.anual, j, 1] <- i
+        esquema.temporadas[i + diferencia.anual, j, 2] <- semana.absoluta(i, semana.inicio)
+        esquema.temporadas[i + diferencia.anual, j, 3] <- datos[i, j]
       }
     }
-
-    diferencia<-inicio.epidemia.esquema-inicio.medio
+    
+    diferencia.global <- inicio.epidemia.esquema - inicio.centrado
     for (i in 1:semanas){
-      if ((i+diferencia)>=1 & (i+diferencia)<=longitud.esquema){
-        esquema.temporadas[i+diferencia,anios+1,1]<-i
-        esquema.temporadas[i+diferencia,anios+1,2]<-semana.absoluta(i,semana.inicio)
+      if ((i+diferencia.global) >= 1 & (i+diferencia.global) <= longitud.esquema){
+        esquema.temporadas[i+diferencia.global, anios+1, 1] <- i
+        esquema.temporadas[i+diferencia.global, anios+1, 2] <- semana.absoluta(i,semana.inicio)
+        esquema.temporadas[i+diferencia.global, anios+1, 3] <- i
       }
     }
-
-    esquema.temporadas[,anios+2,c(1,2)]<-3
-    esquema.temporadas[1:(inicio.epidemia.esquema-1),anios+2,1]<-1
-    esquema.temporadas[(inicio.epidemia.esquema):(inicio.epidemia.esquema+duracion.media-1),anios+2,1]<-2
-    esquema.temporadas[1:(inicio.epidemia.esquema-1),anios+2,2]<-1
-    esquema.temporadas[(inicio.epidemia.esquema):(inicio.epidemia.esquema+duracion.media-1),anios+2,2]<-2
-
+    
+    esquema.temporadas[,anios+2,]<-3
+    esquema.temporadas[1:(diferencia.global + inicio.medio-1),anios+2,]<-1
+    esquema.temporadas[(diferencia.global + inicio.medio):(diferencia.global + inicio.medio + duracion.media-1),anios+2,]<-2
+    # esquema.temporadas[1:(inicio.epidemia.esquema-1),anios+2,2]<-1
+    # esquema.temporadas[(inicio.epidemia.esquema):(inicio.epidemia.esquema+duracion.media-1),anios+2,2]<-2
+    
     ## Temporadas moviles
-
-    temporadas.moviles<-esquema.temporadas[,c(1:anios),3]
-
+    
+    temporadas.moviles <- esquema.temporadas[,c(1:anios),3]
+    
     ## Limites de la temporada
-
-    limites.temporada<-c(inicio.medio,inicio.medio+duracion.media-1)
-    limites.temporada<-rbind(limites.temporada,c(semana.absoluta(inicio.medio,semana.inicio),semana.absoluta(inicio.medio+duracion.media-1,semana.inicio)))
-
+    
+    # limites.temporada<-c(inicio.medio,inicio.medio+duracion.media-1)
+    # limites.temporada<-rbind(limites.temporada,c(semana.absoluta(inicio.medio,semana.inicio),semana.absoluta(inicio.medio+duracion.media-1,semana.inicio)))
+    
     # Limites relativos
-
-    limites.esquema<-c(inicio.epidemia.esquema,inicio.epidemia.esquema+duracion.media-1)
-
+    
+    # limites.esquema<-c(inicio.epidemia.esquema,inicio.epidemia.esquema+duracion.media-1)
+    
     ## En ttotal reordenamos la gripe para q todas las temporadas comiencen la semana "estinicio" (es decir, quitamos informacion
     ## por delante y por detras de esquema.temporada[,,3] para que queden 35 semanas (las que hay).
-
-    temporadas.moviles.recortada<-array(dim=c(semanas,anios))
-
+    
+    temporadas.moviles.recortada<-array(dim=c(semanas,anios), dimnames = list("dummy weeks"=as.character(1:semanas), 
+                                                                              "seasons"=names(datos)))
+    
     ## Temporada gripal comienza "estinicio" y termina en "estinicio+temporada-1"
-
-    diferencia<-inicio.medio-limites.esquema[1]
-
+    
     for (j in 1:anios){
       for (i in 1:semanas){
-        if ((i-diferencia)>=1 & (i-diferencia)<=longitud.esquema){
-          temporadas.moviles.recortada[i,j]<-temporadas.moviles[i-diferencia,j]
+        if ((i+diferencia.global)>=1 & (i+diferencia.global)<=longitud.esquema){
+          temporadas.moviles.recortada[i,j]<-temporadas.moviles[i+diferencia.global,j]
         }else{
           temporadas.moviles.recortada[i,j]<-NA
         }
       }
     }
-
+    
     ## Dibujamos la curva epidemica tipo
-
+    
     # Importante: ?Quitamos los ceros?
     # iy<-(is.na(temporadas.moviles.recortada) | temporadas.moviles.recortada==0)
     iy<-is.na(temporadas.moviles.recortada)
     temporadas.moviles.recortada.no.ceros<-temporadas.moviles.recortada
     temporadas.moviles.recortada.no.ceros[iy]<-NA
     curva.tipo<-t(apply(temporadas.moviles.recortada.no.ceros,1,iconfianza,nivel=i.level.curve,tipo=i.type.curve,ic=T,tipo.boot=i.type.boot,iteraciones.boot=i.iter.boot,colas=2))
-
+    
     ## Seleccionamos los periodos pre, epidemia, y post
-
+    
     ## PRE y POST-TEMPORADA GRIPAL
-
+    
     ## Como no se registra mas q de la semana 40 a la 20, tenemos q muchas de las semanas tienen tasa 0, eliminamos esas semanas,
     ## ya q podrian llevar a una infraestimacion de la tasa base fuera de temporada
-
+    
     pre.datos<-as.vector(as.matrix(extraer.datos.pre.epi(optimo)))
     post.datos<-as.vector(as.matrix(extraer.datos.post.epi(optimo)))
     epi.datos<-as.vector(as.matrix(extraer.datos.epi(optimo)))
     epi.datos.2<-as.vector(as.matrix(apply(datos,2,max.n.valores,n.max=n.max)))
-
+    
     pre.post.datos<-rbind(pre.datos,post.datos)
-
+    
     # IC de la linea basica de pre y post temporada
-
+    
     # por defecto estaba la geometrica
     # pre.d<-pre.datos[!(is.na(pre.datos) | pre.datos==0)]
     # post.d<-post.datos[!(is.na(post.datos) | post.datos==0)]
@@ -388,22 +422,24 @@ memmodel<-function(i.data,
     post.d<-post.datos[!is.na(post.datos)]
     epi.d<-epi.datos[!is.na(epi.datos)]
     epi.d.2<-epi.datos.2[!is.na(epi.datos.2)]
-
+    
     pre.i<-iconfianza(pre.d,nivel=i.level.threshold,tipo=i.type.threshold,ic=T,tipo.boot=i.type.boot,iteraciones.boot=i.iter.boot,colas=i.tails.threshold)
     post.i<-iconfianza(post.d,nivel=i.level.threshold,tipo=i.type.threshold,ic=T,tipo.boot=i.type.boot,iteraciones.boot=i.iter.boot,colas=i.tails.threshold)
     epi.intervalos<-numeric()
     for (niv in i.level.intensity) epi.intervalos<-rbind(epi.intervalos,c(niv,iconfianza(epi.d,nivel=niv,tipo=i.type.intensity,ic=T,colas=i.tails.intensity)))
     epi.intervalos.2<-numeric()
     for (niv in i.level.intensity) epi.intervalos.2<-rbind(epi.intervalos.2,c(niv,iconfianza(epi.d.2,nivel=niv,tipo=i.type.intensity,ic=T,colas=i.tails.intensity)))
-
+    
     pre.post.intervalos<-rbind(pre.i,post.i)
-
+    
     memmodel.output<-list(pre.post.intervals=pre.post.intervalos,
                           ci.length=ic.duracion,
                           ci.percent=ic.porcentaje,
                           ci.start=ic.inicio,
                           mean.length=duracion.media,
                           mean.start=inicio.medio,
+                          centered.length=duracion.centrado,
+                          centered.start=inicio.centrado,
                           moving.epidemics=temporadas.moviles.recortada,
                           epi.intervals=epi.intervalos,
                           epi.intervals.full=epi.intervalos.2,
@@ -442,14 +478,15 @@ memmodel<-function(i.data,
                           param.level.other=i.level.other,
                           param.method=i.method,
                           param.param=i.param,
+                          param.centering=i.centering,
                           param.n.max=i.n.max,
                           param.type.boot=i.type.boot,
                           param.iter.boot=i.iter.boot)
-
+    
     memmodel.output$call<-match.call()
     class(memmodel.output)<-"mem"
   }
-
+  
   return(memmodel.output)
 }
 
@@ -516,7 +553,7 @@ summary.mem<-function(object, ...){
 plot.mem<-function(x,...){
   #opar<-par(mfrow=c(1,2))
   opar<-par(mfrow=c(1,2),mar=c(4,3,1,2)+0.1,mgp=c(3,0.5,0),xpd=T)
-
+  
   # Graph 1
   semanas<-dim(x$data)[1]
   anios<-dim(x$data)[2]
@@ -574,17 +611,17 @@ plot.mem<-function(x,...){
   ya<-otick$range[2]
   if ((i.temporada-1)<=(semanas-f.temporada)) xa<-f.temporada+1 else xa<-1
   legend(x=xa,y=ya,inset=c(0,0),xjust=0,seg.len=1,
-           legend=names.seasons,
-           bty="n",
-           lty=tipos,
-           lwd=anchos,
-           col=colores,
-           cex=0.75,
+         legend=names.seasons,
+         bty="n",
+         lty=tipos,
+         lwd=anchos,
+         col=colores,
+         cex=0.75,
          x.intersp=0.5,
          y.intersp=0.6,
-           text.col="#000000",
-           ncol=1)
-
+         text.col="#000000",
+         ncol=1)
+  
   # Graph 2
   # lineas.basicas<-rbind(matrix(rep(x$pre.post.intervals[1,1:3],i.temporada-1),ncol=3,byrow=T),
   #                       matrix(rep(NA,3*(f.temporada-i.temporada+1)),ncol=3),
@@ -632,8 +669,8 @@ plot.mem<-function(x,...){
   # tamanios<-c(0.75,rep(0.75,n.niveles))
   # colores<-c("#FF0000",colores[3:(3+n.niveles-1)])
   # text(xa,ya,texto,pos=posiciones,col=colores,cex=tamanios)
-
-
+  
+  
   temp1<-memsurveillance(i.current=data.frame(rates=x$typ.curve[,2], row.names = rownames(x$data)),
                          i.epidemic.thresholds = x$epidemic.thresholds,
                          i.intensity.thresholds = x$intensity.thresholds,
@@ -651,7 +688,6 @@ plot.mem<-function(x,...){
                          i.range.y = NA,
                          i.no.labels = F,
                          i.start.end.marks = T)
-
+  
   par(opar)
-}
-
+}
\ No newline at end of file
